package com.qmth.themis.business.templete.impl;

import com.qmth.themis.business.constant.SystemConstant;
import com.qmth.themis.business.dto.response.TEExamStudentDto;
import com.qmth.themis.business.enums.TaskStatusEnum;
import com.qmth.themis.business.excel.ExportUtils;
import com.qmth.themis.business.service.TEExamStudentService;
import com.qmth.themis.business.templete.TaskExportCommon;
import com.qmth.themis.business.templete.TaskExportTemplete;
import com.qmth.themis.common.util.Result;
import com.qmth.themis.common.util.ResultUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.time.LocalDateTime;
import java.util.*;

/**
 * @Description: 考场导出任务
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2020/7/20
 */
@Service
public class TaskExamStudentExportTemplete implements TaskExportTemplete {
    private final static Logger log = LoggerFactory.getLogger(TaskExamStudentExportTemplete.class);

    @Resource
    TEExamStudentService teExamStudentService;

    /**
     * 考生导出模版
     *
     * @param transMap
     * @return
     */
    @Override
    public Result exportTask(Map<String, Object> transMap) throws IOException {
        log.info("开始导出考生数据");
        long start = System.currentTimeMillis();
        TaskExportCommon taskExportCommon = new TaskExportCommon(transMap);
        taskExportCommon.init();
        FileOutputStream fos = null;
        File file = null;
        StringJoiner stringJoiner = null;
        String excelFilePath = null;
        boolean oss = (boolean) taskExportCommon.getOssEnv().get(SystemConstant.OSS);
        try {
            stringJoiner = new StringJoiner("");
            if (!oss) {
                stringJoiner.add(SystemConstant.FILES_DIR).add(File.separator);
            }
            LocalDateTime nowTime = LocalDateTime.now();
            stringJoiner.add(String.valueOf(nowTime.getYear()))
                    .add(File.separator).add(String.format("%02d", nowTime.getMonthValue()))
                    .add(File.separator).add(String.format("%02d", nowTime.getDayOfMonth()));
            file = new File(SystemConstant.TEMP_FILES_DIR + "/" + uuid() + SystemConstant.EXCEL_PREFIX);
            file.createNewFile();
            fos = new FileOutputStream(file);
            Long examId = (Long) transMap.get("examId");
            Long activityId = (Long) transMap.get("activityId");
            String identity = (String) transMap.get("identity");
            String name = (String) transMap.get("name");
            String roomCode = (String) transMap.get("roomCode");
            String courseCode = (String) transMap.get("courseCode");
            String grade = (String) transMap.get("grade");
            Integer enable = (Integer) transMap.get("enable");
            String classNo = (String) transMap.get("classNo");
            Integer hasPhoto = (Integer) transMap.get("hasPhoto");
            List<TEExamStudentDto> list = teExamStudentService.examStudentList(examId, activityId, identity, name, roomCode, courseCode, grade,
                    enable, classNo, hasPhoto);
            ExportUtils.makeExcel(TEExamStudentDto.class, list, fos);
            excelFilePath = taskExportCommon.ossUpload(new FileInputStream(file), stringJoiner.toString());
            taskExportCommon.getTbTaskHistory().setSummary("处理成功");
            long end = System.currentTimeMillis();
            log.info("导出考生数据结束,============耗时============:{}秒", (end - start) / 1000);
        } catch (Exception e) {
            log.error("导出报错", e);
            taskExportCommon.setException(true);
            taskExportCommon.getTbTaskHistory().setSummary("处理出错");
        } finally {
            if (oss && file != null) {
                file.delete();
            }
            taskExportCommon.getTbTaskHistory().setFinishTime(System.currentTimeMillis());
            taskExportCommon.getTbTaskHistory().setStatus(TaskStatusEnum.FINISH);
            taskExportCommon.getTbTaskHistory().setProgress(100.0d);
            //这里写入txt文件
            taskExportCommon.writeExportResultTxt(stringJoiner.toString(), "", excelFilePath);
        }
        return ResultUtil.ok(Collections.singletonMap(SystemConstant.SUCCESS, true));
    }

    private String uuid() {
        return UUID.randomUUID().toString().replaceAll("-", "");
    }
}
